/* Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.flowable.app.rest.editor;

import java.util.ArrayList;
import java.util.List;

import org.flowable.app.model.common.GroupRepresentation;
import org.flowable.app.model.common.RemoteGroup;
import org.flowable.app.model.common.ResultListDataRepresentation;
import org.flowable.app.service.idm.RemoteIdmService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

/**
 * Rest resource for managing groups, used in the editor app.
 */
@RestController
public class EditorGroupsResource {

  @Autowired
  protected RemoteIdmService remoteIdmService;
  
  @RequestMapping(value = "/rest/editor-groups", method = RequestMethod.GET)
  public ResultListDataRepresentation getGroups(@RequestParam(required = false, value = "filter") String filter) {
    List<GroupRepresentation> result = new ArrayList<GroupRepresentation>();
    List<RemoteGroup> groups = remoteIdmService.findGroupsByNameFilter(filter);
    for (RemoteGroup group : groups) {
      result.add(new GroupRepresentation(group));
    }
    return new ResultListDataRepresentation(result);
  }
}
